@@ -1,13 +1,14 @@
 ## ----setup, include=FALSE-----------------------------------------------------
 library(INLA)
-inla.setOption(num.threads="1:1")
-inla.setOption(smtp="taucs")
-if (file.exists("myinit.R")) source("myinit.R")
-
 library(knitr)
 library(formatR)
+library(fastGHQuad)
+library(pracma)
+inla.setOption(smtp="taucs")
+inla.setOption(num.threads="1:1")
+if (file.exists("myinit.R")) source("myinit.R")
 set.seed(20202)
-opts_chunk$set(echo = TRUE)
+knitr::opts_chunk$set(echo = TRUE)
 opts_chunk$set(size='small'
                , cache=FALSE
                , cache.path='cache/group-cv'
@@ -16,7 +17,21 @@ opts_chunk$set(size='small'
                , message=FALSE
                , fig.align='center'
                , fig.path='figures/group-cv/'
-               , fig.pos='H')
+               , fig.pos='H'
+               , background='#ffffff'
+               , results='hold'
+               , eval=TRUE)
+
+# gh.x = c(-6.995680e+00,-6.275079e+00,-5.673961e+00,-5.133596e+00,-4.631560e+00,-4.156272e+00,-3.700743e+00,-3.260321e+00,-2.831680e+00,-2.412318e+00,-2.000259e+00,-1.593886e+00,-1.191827e+00,-7.928770e-01,-3.959427e-01,-2.722499e-16,3.959427e-01,7.928770e-01,1.191827e+00,1.593886e+00,2.000259e+00,2.412318e+00,2.831680e+00,3.260321e+00,3.700743e+00,4.156272e+00,4.631560e+00,5.133596e+00,5.673961e+00,6.275079e+00,6.995680e+00)
+# gh.w = c(4.618968e-22,5.110609e-18,5.899556e-15,1.860374e-12,2.352492e-10,1.461199e-08,5.043713e-07,1.049860e-05,1.395209e-04,1.233683e-03,7.482800e-03,3.184723e-02,9.671795e-02,2.121328e-01,3.387727e-01,3.957786e-01,3.387727e-01,2.121328e-01,9.671795e-02,3.184723e-02,7.482800e-03,1.233683e-03,1.395209e-04,1.049860e-05,5.043713e-07,1.461199e-08,2.352492e-10,1.860374e-12,5.899556e-15,5.110609e-18,4.618968e-22)
+
+gh.x = fastGHQuad::gaussHermiteData(51)$x
+gh.w = fastGHQuad::gaussHermiteData(51)$w
+
+inla.lik.pred = function(y,args = NULL,args.string,eta.mean,eta.sd,eta.skewness){
+    res = dnorm(y,mean = eta.mean,sd = sqrt(eta.sd^2 + 1/exp(args$theta)))
+    return(res)
+}
 
 ## ----message=FALSE, warning=FALSE,echo = FALSE--------------------------------
 n_rat = 10
@@ -36,48 +51,63 @@ res = inla(formula = formula,
            family = "normal"
            )
 
-## ----message = FALSE, warning=FALSE, echo = FALSE-----------------------------
-res = inla(formula = formula,
-           data = data.frame(y = c(y,rep(NA,n_rat+1)), id = c(id,1:n_rat,n_rat+1)),
-           family = "normal",
-           control.compute = list(config = TRUE)
-           )
-
-nconfig = res$misc$configs$nconfig
-weights = unlist(lapply(1:nconfig,FUN = function(i){res$misc$configs$config[[i]]$log.posterior}))
-weights = exp(weights)/sum(exp(weights))
-eps = 0.001
-q = seq(eps,1-eps,eps)
-preddensity = vector(mode = "list",length = n_rat+1)
-
-for(rat_idx in c(1,11)){
-    predictor_idx = n + rat_idx
-    mode_idx = which.max(weights)
-    sd = 1/sqrt(exp(res$misc$configs$config[[mode_idx]]$theta[1]))
-    mode_eta_mu = res$misc$configs$config[[mode_idx]]$Predictor[predictor_idx,"mean"]
-    mode_eta_sd = sqrt(res$misc$configs$config[[mode_idx]]$Predictor[predictor_idx,"variance"])
-    eta_eval = mode_eta_mu + seq(-5,5)*mode_eta_sd
-    y_eval = qnorm(q,mean = mode_eta_mu,sd = sqrt(mode_eta_sd^2 + sd^2))
-    
-    preddensity_conditional = list(mode = "list",length = nconfig)
-    preddensity[[rat_idx]]$y_eval = y_eval
-    preddensity[[rat_idx]]$density = numeric(length(y_eval))
-    for(config_idx in 1:nconfig){
-        sd = 1/sqrt(exp(res$misc$configs$config[[config_idx]]$theta[1]))
-        preddensity_conditional[[config_idx]] = dnorm(y_eval,mean = res$misc$configs$config[[config_idx]]$Predictor[predictor_idx,"mean"],sd = sqrt(sqrt(res$misc$configs$config[[config_idx]]$Predictor[predictor_idx,"variance"])^2 + sd^2))
-        preddensity[[rat_idx]]$density  = preddensity[[rat_idx]]$density +  weights[config_idx]*preddensity_conditional[[config_idx]]
-    }
-}
-plot(preddensity[[1]]$y_eval,preddensity[[1]]$density,
-     main = "Predictive Density of A Measure from Rat 1",
-     xlab = "weight",
-     ylab = "density",
-     type="l")
-plot(preddensity[[11]]$y_eval,preddensity[[11]]$density,
-     main = "Predictive Density of A Measure from An Unobserved Rat",
-     xlab = "weight",
-     ylab = "density",
-     type="l")
+## ----message = FALSE, warning=FALSE, echo = TRUE, eval=FALSE------------------
+# ##
+# ## this example needs to be fixed so it does not depend on the removed
+# ## inla.likelihood* feature
+# ##
+# res = inla(formula = formula,
+#            data = data.frame(y = c(y,rep(NA,n_rat+1)), id = c(id,1:n_rat,n_rat+1)),
+#            family = "normal",
+#            control.compute = list(config = TRUE,likelihood.info = T)
+#            )
+# nconfig = res$misc$configs$nconfig
+# weights = unlist(lapply(1:nconfig,FUN = function(i){res$misc$configs$config[[i]]$log.posterior}))
+# weights = exp(weights)/sum(exp(weights))
+# eps = 0.001
+# q = seq(eps,1-eps,eps)
+# preddensity = vector(mode = "list",length = n_rat+1)
+# for(rat_idx in c(1,11)){
+#     predictor_idx = n + rat_idx
+#     mode_idx = which.max(weights)
+#     args = INLA:::inla.likelihood.parser(res$misc$configs$config[[1]]$arg.str[1])
+#     for(mode_idx in mode_idx){
+#         args$theta = res$misc$configs$config[[mode_idx]]$theta[1]
+#         mode_eta_mu = res$misc$configs$config[[mode_idx]]$Predictor[predictor_idx,"mean"]
+#         mode_eta_sd = sqrt(res$misc$configs$config[[mode_idx]]$Predictor[predictor_idx,"variance"])
+#         eta_eval = mode_eta_mu + seq(-5,5)*mode_eta_sd
+#         args$linear.predictor = eta_eval
+#         qfunc = inla.likelihood(type = "q",args)
+#         y_eval = c()
+#         for(i in 1:length(q)){
+#             y_eval = c(y_eval,qfunc(q[i]))
+#         }
+#     }
+#     y_eval = sort(y_eval)
+# 
+#     preddensity_conditional = list(mode = "list",length = nconfig)
+#     preddensity[[rat_idx]]$y_eval = y_eval
+#     preddensity[[rat_idx]]$density = numeric(length(y_eval))
+#     for(config_idx in 1:nconfig){
+#         args$theta = res$misc$configs$config[[config_idx]]$theta[1]
+#         preddensity_conditional[[config_idx]] = inla.lik.pred(y_eval,
+#                                                   args = args,
+#                                                   eta.mean = res$misc$configs$config[[config_idx]]$Predictor[predictor_idx,"mean"],
+#                                                   eta.sd = sqrt(res$misc$configs$config[[config_idx]]$Predictor[predictor_idx,"variance"])
+#                                                   )
+#         preddensity[[rat_idx]]$density  = preddensity[[rat_idx]]$density +  weights[config_idx]*preddensity_conditional[[config_idx]]
+#     }
+# }
+# plot(preddensity[[1]]$y_eval,preddensity[[1]]$density,
+#      main = "Predictive Density of A Measure from Rat 1",
+#      xlab = "weight",
+#      ylab = "density",
+#      type="l")
+# plot(preddensity[[11]]$y_eval,preddensity[[11]]$density,
+#      main = "Predictive Density of A Measure from An Unobserved Rat",
+#      xlab = "weight",
+#      ylab = "density",
+#      type="l")
 
 ## ----message = FALSE, warning=FALSE-------------------------------------------
 loocv_res = inla.group.cv(result = res)
@@ -129,7 +159,7 @@ formula = y ~ 1 + x + f(id, model = "iid") + f(id2, W = x,model = "iid")
 res = inla(formula = formula,
            data = data,
            family = "normal",
-           control.compute = list(config = TRUE)
+           control.compute = list(config = TRUE,likelihood.info = T)
            )
 
 ## ----message = FALSE, warning=FALSE-------------------------------------------
@@ -137,7 +167,7 @@ formula_alt =  y ~ 1 + x + f(id, model = "iid")
 res_alt = inla(formula = formula_alt,
            data = data,
            family = "normal",
-           control.compute = list(config = TRUE)
+           control.compute = list(config = TRUE,likelihood.info = T)
            )
 
 ## ----message = FALSE, warning=FALSE-------------------------------------------
@@ -186,10 +216,10 @@ y_observed = y[testing.point]
 # 2. get the hyperparameter density
 nconfigs = res$misc$configs$nconfig
 weights = unlist(lapply(X = 1:nconfigs,
-                        FUN = function(config.idx){
-                            res$misc$configs$config[[config.idx]]$log.posterior 
-                            + res$misc$configs$config[[config.idx]]$gcpodens.moments[testing.point,3]
-                            }))
+	FUN = function(config.idx){
+    res$misc$configs$config[[config.idx]]$log.posterior 
+    + res$misc$configs$config[[config.idx]]$gcpodens.moments[testing.point,3]
+    }))
 weights = exp(weights)/sum(exp(weights))
 #3. get samples of y_i|y_{-I_i},theta
 n = 1e6
@@ -197,9 +227,9 @@ n_theta = as.numeric(rmultinom(n = 1,size = n,prob = weights))
 y_sample = c()
 for(config.idx in 1:nconfigs){
     if(n_theta[config.idx]>0){
-         eta = rnorm(n_theta[config.idx],
-                     mean = res$misc$configs$config[[config.idx]]$gcpodens.moments[testing.point,1],
-                     sd =  sqrt(res$misc$configs$config[[config.idx]]$gcpodens.moments[testing.point,2]))
+		eta = rnorm(n_theta[config.idx],
+			mean = res$misc$configs$config[[config.idx]]$gcpodens.moments[testing.point,1],
+			sd = sqrt(res$misc$configs$config[[config.idx]]$gcpodens.moments[testing.point,2]))
          sd = sqrt(1/exp(res$misc$configs$config[[config.idx]]$theta[1]))
          y_sample_config = rnorm(n = n_theta[config.idx],mean = eta,sd = sd)
          y_sample = c(y_sample,y_sample_config)
